"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocketSubscriber = void 0;
var operators_1 = require("rxjs/operators");
var xcWebSocketBridgeConfiguration_1 = require("../configuration/xcWebSocketBridgeConfiguration");
var apiConfiguration_1 = require("../configuration/apiConfiguration");
require("rxjs/add/operator/toPromise");
var xcomponentMessages_1 = require("./xcomponentMessages");
var StateMachineInstance_1 = require("../interfaces/StateMachineInstance");
var uuid = require("uuid/v4");
var log4ts_1 = require("log4ts");
var WebSocketSubscriber = /** @class */ (function () {
    function WebSocketSubscriber(webSocketWrapper, configuration) {
        this.webSocketWrapper = webSocketWrapper;
        this.configuration = configuration;
        this.logger = log4ts_1.Logger.getLogger('WebSocketSubscriber');
        this.subscribedStateMachines = {};
        this.deserializer = new xcomponentMessages_1.Deserializer();
        this.serializer = new xcomponentMessages_1.Serializer();
        this.timeout = '00:00:10';
        var thisSubscriber = this;
        var webSocketUpdates$ = this.webSocketWrapper.getObservable();
        if (webSocketUpdates$ !== undefined && webSocketUpdates$ !== null) {
            this.updates$ = webSocketUpdates$.pipe((0, operators_1.map)(function (rawMessage) {
                return thisSubscriber.deserializer.deserialize(rawMessage.data || rawMessage);
            }));
        }
    }
    WebSocketSubscriber.prototype.setStateMachineRefSendPublisher = function (stateMachineRefSendPublisher) {
        this.stateMachineRefSendPublisher = stateMachineRefSendPublisher;
    };
    WebSocketSubscriber.prototype.getSnapshot = function (componentName, stateMachineName, privateTopics) {
        var replyTopic = uuid();
        var thisSubscriber = this;
        var promise = this.updates$
            .pipe((0, operators_1.filter)(function (data) {
            return data.command === xcWebSocketBridgeConfiguration_1.Commands[xcWebSocketBridgeConfiguration_1.Commands.snapshot] && data.topic === replyTopic;
        }), (0, operators_1.first)(), (0, operators_1.map)(function (data) {
            thisSubscriber.sendUnsubscribeRequestToTopic(replyTopic, xcWebSocketBridgeConfiguration_1.Kinds.Snapshot);
            return thisSubscriber.getJsonDataFromSnapshot(data.stringData, data.topic);
        }))
            .toPromise();
        this.sendSubscribeRequestToTopic(replyTopic, xcWebSocketBridgeConfiguration_1.Kinds.Snapshot);
        var dataToSendSnapshot = this.getDataToSendSnapshot(componentName, stateMachineName, replyTopic, privateTopics);
        this.webSocketWrapper.send(thisSubscriber.serializer.convertToWebsocketInputFormat(dataToSendSnapshot));
        return promise;
    };
    WebSocketSubscriber.prototype.getDataToSendSnapshot = function (componentName, stateMachineName, replyTopic, privateTopics) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        var topic = this.configuration.getSnapshotTopic(componentCode);
        var jsonMessage = {
            Timeout: this.timeout,
            CallerPrivateTopic: privateTopics.getSubscriberTopics(),
            ReplyTopic: replyTopic
        };
        var header = (0, xcomponentMessages_1.getHeaderWithIncomingType)();
        header.ComponentCode = componentCode;
        header.StateMachineCode = stateMachineCode;
        var dataToSendSnapshot = {
            RoutingKey: topic,
            ComponentCode: componentCode,
            Event: {
                Header: header,
                JsonMessage: JSON.stringify(jsonMessage)
            }
        };
        return dataToSendSnapshot;
    };
    WebSocketSubscriber.prototype.prepareStateMachineUpdates = function (componentName, stateMachineName) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        var thisSubscriber = this;
        var filteredObservable = this.updates$.pipe((0, operators_1.filter)(function (data) { return data.command === xcWebSocketBridgeConfiguration_1.Commands[xcWebSocketBridgeConfiguration_1.Commands.update]; }), (0, operators_1.map)(function (data) { return thisSubscriber.getJsonDataFromEvent(data.stringData, data.topic); }), (0, operators_1.filter)(function (jsonData) {
            return thisSubscriber.isSameComponent(jsonData, componentCode) &&
                thisSubscriber.isSameStateMachine(jsonData, stateMachineCode);
        }));
        return filteredObservable;
    };
    WebSocketSubscriber.prototype.isSameComponent = function (jsonData, componentCode) {
        var sameComponent = jsonData.stateMachineRef.ComponentCode === componentCode;
        return sameComponent;
    };
    WebSocketSubscriber.prototype.isSameStateMachine = function (jsonData, stateMachineCode) {
        var sameStateMachine = jsonData.stateMachineRef.StateMachineCode === stateMachineCode;
        return sameStateMachine;
    };
    WebSocketSubscriber.prototype.getStateMachineUpdates = function (componentName, stateMachineName) {
        var filteredObservable = this.prepareStateMachineUpdates(componentName, stateMachineName);
        this.sendSubscribeRequest(componentName, stateMachineName);
        return filteredObservable;
    };
    WebSocketSubscriber.prototype.canSubscribe = function (componentName, stateMachineName) {
        var componentCode = this.configuration.getComponentCode(componentName);
        var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
        return this.configuration.containsSubscriber(componentCode, stateMachineCode, apiConfiguration_1.SubscriberEventType.Update);
    };
    WebSocketSubscriber.prototype.subscribe = function (componentName, stateMachineName, stateMachineUpdateListener) {
        this.prepareStateMachineUpdates(componentName, stateMachineName).subscribe(function (stateMachineInstance) {
            stateMachineUpdateListener.onStateMachineUpdate(stateMachineInstance);
        });
        this.sendSubscribeRequest(componentName, stateMachineName);
    };
    WebSocketSubscriber.prototype.sendSubscribeRequest = function (componentName, stateMachineName) {
        if (!this.isSubscribed(this.subscribedStateMachines, componentName, stateMachineName)) {
            var componentCode = this.configuration.getComponentCode(componentName);
            var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
            var topic = this.configuration.getSubscriberTopic(componentCode, stateMachineCode, apiConfiguration_1.SubscriberEventType.Update);
            var kind = xcWebSocketBridgeConfiguration_1.Kinds.Public;
            this.sendSubscribeRequestToTopic(topic, kind);
            this.addSubscribedStateMachines(componentName, stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.sendSubscribeRequestToTopic = function (topic, kind) {
        var data = this.getDataToSend(topic, kind);
        var commandData = {
            Command: xcWebSocketBridgeConfiguration_1.Commands[xcWebSocketBridgeConfiguration_1.Commands.subscribe],
            Data: data
        };
        var input = this.serializer.convertCommandDataToWebsocketInputFormat(commandData);
        this.webSocketWrapper.send(input);
    };
    WebSocketSubscriber.prototype.sendUnsubscribeRequestToTopic = function (topic, kind) {
        var data = this.getDataToSend(topic, kind);
        var commandData = {
            Command: xcWebSocketBridgeConfiguration_1.Commands[xcWebSocketBridgeConfiguration_1.Commands.unsubscribe],
            Data: data
        };
        var input = this.serializer.convertCommandDataToWebsocketInputFormat(commandData);
        this.webSocketWrapper.send(input);
    };
    WebSocketSubscriber.prototype.getDataToSend = function (topic, kind) {
        return {
            Header: (0, xcomponentMessages_1.getHeaderWithIncomingType)(),
            JsonMessage: JSON.stringify({
                Topic: {
                    Key: topic,
                    kind: kind
                }
            })
        };
    };
    WebSocketSubscriber.prototype.unsubscribe = function (componentName, stateMachineName) {
        if (this.isSubscribed(this.subscribedStateMachines, componentName, stateMachineName)) {
            var componentCode = this.configuration.getComponentCode(componentName);
            var stateMachineCode = this.configuration.getStateMachineCode(componentName, stateMachineName);
            var topic = this.configuration.getSubscriberTopic(componentCode, stateMachineCode, apiConfiguration_1.SubscriberEventType.Update);
            var kind = xcWebSocketBridgeConfiguration_1.Kinds.Public;
            var data = this.getDataToSend(topic, kind);
            var commandData = {
                Command: xcWebSocketBridgeConfiguration_1.Commands[xcWebSocketBridgeConfiguration_1.Commands.unsubscribe],
                Data: data
            };
            this.webSocketWrapper.send(this.serializer.convertCommandDataToWebsocketInputFormat(commandData));
            this.removeSubscribedStateMachines(componentName, stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.isSubscribed = function (subscribedStateMachines, componentName, stateMachineName) {
        var isSubscribed = subscribedStateMachines[componentName] !== undefined &&
            subscribedStateMachines[componentName].indexOf(stateMachineName) > -1;
        return isSubscribed;
    };
    WebSocketSubscriber.prototype.addSubscribedStateMachines = function (componentName, stateMachineName) {
        if (this.subscribedStateMachines[componentName] === undefined) {
            this.subscribedStateMachines[componentName] = [stateMachineName];
        }
        else {
            this.subscribedStateMachines[componentName].push(stateMachineName);
        }
    };
    WebSocketSubscriber.prototype.removeSubscribedStateMachines = function (componentName, stateMachineName) {
        var index = this.subscribedStateMachines[componentName].indexOf(stateMachineName);
        this.subscribedStateMachines[componentName].splice(index, 1);
    };
    WebSocketSubscriber.prototype.getJsonDataFromSnapshot = function (data, topic) {
        this.logger.debug('JsonData received from snapshot: ', { data: data, topic: topic }, 2);
        var jsonData = this.deserializer.getJsonData(data);
        var b64Data = JSON.parse(jsonData.JsonMessage).Items;
        var items;
        try {
            items = JSON.parse(this.deserializer.decodeServerMessage(b64Data));
        }
        catch (e) {
            items = b64Data;
        }
        var snapshotItems = new Array();
        for (var i = 0; i < items.length; i++) {
            var stateMachineRef = this.getStateMachineRef(items[i].StateMachineId, parseInt(items[i].WorkerId, undefined), parseInt(items[i].ComponentCode, undefined), parseInt(items[i].StateMachineCode, undefined), parseInt(items[i].StateCode, undefined));
            snapshotItems.push(new StateMachineInstance_1.StateMachineInstance(stateMachineRef, items[i].PublicMember));
        }
        return snapshotItems;
    };
    WebSocketSubscriber.prototype.getJsonDataFromEvent = function (data, topic) {
        this.logger.debug('JsonData received from event: ', { data: data, topic: topic }, 2);
        var jsonData = this.deserializer.getJsonData(data);
        var stateMachineRef = this.getStateMachineRef(jsonData.Header.StateMachineId, jsonData.Header.WorkerId, jsonData.Header.ComponentCode, jsonData.Header.StateMachineCode, jsonData.Header.StateCode, jsonData.Header.ErrorMessage);
        return new StateMachineInstance_1.StateMachineInstance(stateMachineRef, JSON.parse(jsonData.JsonMessage));
    };
    WebSocketSubscriber.prototype.getStateMachineRef = function (StateMachineId, workerId, componentCode, stateMachineCode, stateCode, errorMessage) {
        var thisSubscriber = this;
        var stateMachineRef = {
            StateMachineId: StateMachineId,
            WorkerId: workerId,
            ComponentCode: componentCode,
            StateMachineCode: stateMachineCode,
            StateName: errorMessage
                ? xcomponentMessages_1.fatalErrorState
                : thisSubscriber.configuration.getStateName(componentCode, stateMachineCode, stateCode),
            send: function (messageType, jsonMessage, visibilityPrivate, specifiedPrivateTopic) {
                thisSubscriber.stateMachineRefSendPublisher.sendWithStateMachineRef(stateMachineRef, messageType, jsonMessage, visibilityPrivate, specifiedPrivateTopic);
            },
            ErrorMessage: errorMessage
        };
        return stateMachineRef;
    };
    return WebSocketSubscriber;
}());
exports.WebSocketSubscriber = WebSocketSubscriber;
//# sourceMappingURL=WebSocketSubscriber.js.map