"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultApiConfiguration = exports.SubscriberEventType = void 0;
var SubscriberEventType;
(function (SubscriberEventType) {
    SubscriberEventType[SubscriberEventType["Update"] = 0] = "Update";
    SubscriberEventType[SubscriberEventType["Error"] = 1] = "Error";
})(SubscriberEventType || (exports.SubscriberEventType = SubscriberEventType = {}));
var DefaultApiConfiguration = /** @class */ (function () {
    function DefaultApiConfiguration(rawConfig) {
        this._config = rawConfig;
        if (!this.validate()) {
            throw new Error('invalid configuration');
        }
    }
    DefaultApiConfiguration.prototype.validate = function () {
        return (this._config !== undefined &&
            this._config.deployment !== undefined &&
            this._config.deployment.codesConverter !== undefined &&
            this._config.deployment.clientAPICommunication !== undefined);
    };
    DefaultApiConfiguration.prototype.getComponentCode = function (componentName) {
        var component = this.findComponentByName(componentName);
        return Number(component.attributes.id);
    };
    DefaultApiConfiguration.prototype.getStateMachineCode = function (componentName, stateMachineName) {
        var component = this.findComponentByName(componentName);
        var stateMachine = this.findStateMachineByName(component, stateMachineName);
        return Number(stateMachine.attributes.id);
    };
    DefaultApiConfiguration.prototype.findStateMachineByName = function (component, stateMachineName) {
        var stateMachine = this.findStateMachine(component, function (stm) { return stm.attributes.name === stateMachineName; });
        if (!stateMachine) {
            throw new Error("StateMachine '".concat(stateMachineName, "' not found"));
        }
        return stateMachine;
    };
    DefaultApiConfiguration.prototype.findStateMachineByCode = function (component, stateMachineCode) {
        var stateMachine = this.findStateMachine(component, function (stm) { return Number(stm.attributes.id) === stateMachineCode; });
        if (!stateMachine) {
            throw new Error("StateMachine '".concat(stateMachineCode, "' not found"));
        }
        return stateMachine;
    };
    DefaultApiConfiguration.prototype.findStateByCode = function (stateMachine, stateCode) {
        var result = stateMachine.states[0].State.find(function (state) { return Number(state.attributes.id) === stateCode; });
        if (!result) {
            throw new Error("State '".concat(stateCode, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponentByName = function (componentName) {
        var result = this.findComponent(function (component) { return component.attributes.name === componentName; });
        if (!result) {
            throw new Error("Component '".concat(componentName, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponentByCode = function (componentCode) {
        var result = this.findComponent(function (component) { return Number(component.attributes.id) === componentCode; });
        if (!result) {
            throw new Error("Component '".concat(componentCode, "' not found"));
        }
        return result;
    };
    DefaultApiConfiguration.prototype.findComponent = function (predicate) {
        return this._config.deployment.codesConverter[0].components[0].component.find(predicate);
    };
    DefaultApiConfiguration.prototype.findStateMachine = function (component, predicate) {
        return component.stateMachines[0].stateMachine.find(predicate);
    };
    DefaultApiConfiguration.prototype.containsComponent = function (componentName) {
        var result = this.findComponent(function (component) { return component.attributes.name === componentName; });
        return result ? true : false;
    };
    DefaultApiConfiguration.prototype.containsStateMachine = function (componentName, stateMachineName) {
        var result = this.findComponent(function (component) { return component.attributes.name === componentName; });
        if (result) {
            return result.stateMachines[0].stateMachine.find(function (stm) { return stm.attributes.name === stateMachineName; }) != null;
        }
        return false;
    };
    DefaultApiConfiguration.prototype.containsPublisher = function (componentCode, stateMachineCode, messageType) {
        return this.getPublisher(componentCode, stateMachineCode, messageType) !== undefined;
    };
    DefaultApiConfiguration.prototype.containsSubscriber = function (componentCode, stateMachineCode, type) {
        return this.getSubscriber(componentCode, stateMachineCode, type) !== undefined;
    };
    DefaultApiConfiguration.prototype.getPublisherDetails = function (componentCode, stateMachineCode, messageType) {
        var publisher = this.getPublisher(componentCode, stateMachineCode, messageType);
        if (!publisher) {
            throw new Error("publisher not found - component code: ".concat(componentCode, " - statemachine code: ").concat(stateMachineCode, " - message type: ").concat(messageType, " "));
        }
        return {
            eventCode: Number(publisher.attributes.eventCode),
            routingKey: publisher.topic[0].value
        };
    };
    DefaultApiConfiguration.prototype.getPublisher = function (componentCode, stateMachineCode, messageType) {
        return this._config.deployment.clientAPICommunication[0].publish.find(function (pub) {
            return Number(pub.attributes.componentCode) === componentCode &&
                Number(pub.attributes.stateMachineCode) === stateMachineCode &&
                pub.attributes.event === messageType;
        });
    };
    DefaultApiConfiguration.prototype.getSubscriberTopic = function (componentCode, stateMachineCode, type) {
        var subscriber = this.getSubscriber(componentCode, stateMachineCode, type);
        if (!subscriber) {
            throw new Error("Subscriber not found - component code: ".concat(componentCode, " - statemachine code: ").concat(stateMachineCode));
        }
        return subscriber.topic[0].value;
    };
    DefaultApiConfiguration.prototype.getSubscriber = function (componentCode, stateMachineCode, type) {
        return this._config.deployment.clientAPICommunication[0].subscribe.find(function (pub) {
            return Number(pub.attributes.componentCode) === componentCode &&
                Number(pub.attributes.stateMachineCode) === stateMachineCode &&
                pub.attributes.eventType === SubscriberEventType[type].toUpperCase();
        });
    };
    DefaultApiConfiguration.prototype.getSnapshotTopic = function (componentCode) {
        var snapshot = this._config.deployment.clientAPICommunication[0].snapshot.find(function (pub) { return Number(pub.attributes.componentCode) === componentCode; });
        if (!snapshot) {
            throw new Error("Snapshot topic not found - component code: ".concat(componentCode));
        }
        return snapshot.topic[0].value;
    };
    DefaultApiConfiguration.prototype.getStateName = function (componentCode, stateMachineCode, stateCode) {
        var component = this.findComponentByCode(componentCode);
        var stateMachine = this.findStateMachineByCode(component, stateMachineCode);
        var state = this.findStateByCode(stateMachine, stateCode);
        return state.attributes.name;
    };
    return DefaultApiConfiguration;
}());
exports.DefaultApiConfiguration = DefaultApiConfiguration;
//# sourceMappingURL=apiConfiguration.js.map